@@ -71,7 +71,7 @@ time_t tstamp ( time_t last, FILE *f )
 
   now = time ( 0 ) ;
 
-  strftime ( tbuf, MAXTSTAMP,  ( now - last > 600 ) ? "%c" : "%M:%S",
+  strftime ( tbuf, MAXTSTAMP,  ( now - last > 600 ) ? "%c" : "%H:%M:%S",
 	     localtime( &now ) ) ;
 #ifdef ENABLE_NLS
   if ( use_utf8 ) {
