@@ -34,6 +34,7 @@ COPYING distributed with the source files.
 #include "utils/thread.h"
 #include "utils/mutex.h"
 #include "utils/shared_handle.h"
+#include "utils/callback.h"
 
 #ifdef HAVE_OSTREAM
 #include <ios>
@@ -183,7 +184,8 @@ void EfaxController::sendfax_impl(const Fax_item& fax_item, bool in_timer_event)
     pthread_sigmask(SIG_BLOCK, &sig_mask, 0);
 
     std::auto_ptr<Thread::Thread> res =
-      Thread::Thread::start(sigc::mem_fun(*this, &EfaxController::make_fax_thread), false);
+      Thread::Thread::start(Callback::make(*this, &EfaxController::make_fax_thread),
+			    false);
     if (!res.get()) {
       write_error("Cannot start thread to make fax for sending\n");
       stdout_pipe.close();
