@@ -20,10 +20,11 @@ COPYING distributed with the source files.
 
 #include <sigc++/sigc++.h>
 
-#include "pipes.h"
-#include "sem_sync.h"
-#include "notifier.h"
-#include "io_watch.h"
+#include "utils/pipes.h"
+#include "utils/sem_sync.h"
+#include "utils/notifier.h"
+#include "utils/io_watch.h"
+#include "utils/utf8_utils.h"
 
 
 struct Fax_item {
@@ -41,6 +42,8 @@ private:
   State state;
   bool close_down;
 
+  int received_fax_count;
+
   SemSync fax_made_sem;
   Notifier fax_made_notify;
   Notifier no_fax_made_notify;
@@ -50,12 +53,14 @@ private:
 
   PipeFifo stdout_pipe;
   guint iowatch_tag;
+  Utf8::Reassembler stdout_pipe_reassembler;
 
-  void clear_receive_dirname(void);
+  void receive_cleanup(void);
+  std::pair<std::string, std::string> save_received_fax(void);
+  std::pair<std::string, std::string> save_sent_fax(void);
   void join_child(void);
   void unjoin_child(void);
   void kill_child(void);
-  void save_sent_fax(void);
   void cleanup_fax_send_fail(void);
   std::vector<std::string> state_messages;
   void make_fax_thread(void);
@@ -64,18 +69,19 @@ private:
   void sendfax_slot(void);
   void no_fax_made_slot(void);
   bool read_pipe_slot(void);
+  void sendfax_impl(const Fax_item&, bool);
 
   std::pair<const char*, char* const*> get_gs_parms(const std::string&);
   std::pair<const char*, char* const*> get_receive_parms(int);
   void delete_parms(std::pair<const char*, char* const*>);
-  void cleanup_on_start(void);
 
   // we don't want to permit copies of this class
   EfaxController(const EfaxController&);
   void operator=(const EfaxController&);
 
 public:
-  sigc::signal0<void> fax_received_notify;
+  sigc::signal1<void, const std::pair<std::string, std::string>&> fax_received_notify;
+  sigc::signal1<void, const std::pair<std::string, std::string>&> fax_sent_notify;
   sigc::signal0<void> ready_to_quit_notify;
   sigc::signal1<void, const char*> stdout_message;
   sigc::signal1<void, const char*> write_state;
@@ -86,10 +92,13 @@ public:
   int get_state(void) const {return state;}
   bool is_receiving_fax(void) const;
 
+  int get_count(void) {return received_fax_count;}
+  void reset_count(void) {received_fax_count = 0; display_state();}
+
   void stop(void);
 
   void efax_closedown(void);
-  void sendfax(const Fax_item&);
+  void sendfax(const Fax_item& fax_item) {sendfax_impl(fax_item, false);}
   void receive(State);
   EfaxController(void);
 };
