@@ -60,6 +60,7 @@ COPYING distributed with the source files.
 #include "utils/thread.h"
 #include "utils/utf8_utils.h"
 #include "utils/toolbar_append_widget.h"
+#include "utils/callback.h"
 
 #ifdef HAVE_OSTREAM
 #include <ios>
@@ -975,16 +976,18 @@ void FaxListDialog::print_fax(void) {
     sigaddset(&sig_mask, SIGHUP);
     pthread_sigmask(SIG_BLOCK, &sig_mask, 0);
 
-    // hand ownership to the new thread - Thread::Thread::start() could throw
-    // in the event of memory allocation failure but as a matter of policy we
-    // do not deal with that condition in this program so we can pass a raw pointer
-    // rather than the std::auto_ptr<> object - anyway passing a std::auto_ptr<>
-    // object wouldn't be enough as we would still have to catch the std::bad_alloc
-    // exception in order to pop() off the object in async_queue
+    // hand ownership of fax_basename_a to the new thread - Thread::Thread::start()
+    // could throw in the event of memory allocation failure, as could the creation of
+    // the Callback2 object on free store, but as a matter of policy we do not deal
+    // with that condition in this program so we can pass a raw pointer rather than
+    // the std::auto_ptr<> object - anyway passing a std::auto_ptr<> object wouldn't
+    // be enough as we would still have to catch the std::bad_alloc exception in
+    // order to pop() off the object in async_queue
     std::string* arg_p = fax_basename_a.release();
     std::auto_ptr<Thread::Thread> res =
-      Thread::Thread::start(sigc::bind(sigc::mem_fun(*this, &FaxListDialog::print_fax_thread),
-				       arg_p, prog_config.gtkprint), false);
+      Thread::Thread::start(Callback::make(*this, &FaxListDialog::print_fax_thread,
+					   arg_p, prog_config.gtkprint),
+			    false);
     if (!res.get()) {
       delete arg_p;
       write_error("Cannot start new print thread, fax will not be printed\n");
@@ -1228,17 +1231,18 @@ void FaxListDialog::view_fax(void) {
     sigaddset(&sig_mask, SIGHUP);
     pthread_sigmask(SIG_BLOCK, &sig_mask, 0);
 
-    // hand ownership to the new thread - Thread::Thread::start() could throw
-    // in the event of memory allocation failure but as a matter of policy we
-    // do not deal with that condition in this program so we can pass a raw pointer
-    // rather than the std::auto_ptr<> object
+    // hand ownership of the file_pair arguments to the new thread - Thread::Thread::start()
+    // could throw in the event of memory allocation failure, as could the creation of
+    // the Callback2 object on free store, but as a matter of policy we do not deal with
+    // that condition in this program so we can pass a raw pointer rather than the
+    // std::auto_ptr<> object
     std::string* fax_basename_p = fax_basename_a.release();
     std::pair<std::string*, int> file_pair;
     file_pair.first = filename_a.release();
     file_pair.second = file_fd;
     std::auto_ptr<Thread::Thread> res =
-      Thread::Thread::start(sigc::bind(sigc::mem_fun(*this, &FaxListDialog::view_fax_thread),
-				       fax_basename_p, file_pair),
+      Thread::Thread::start(Callback::make(*this, &FaxListDialog::view_fax_thread,
+					   fax_basename_p, file_pair),
 			    false);
     if (!res.get()) {
       delete fax_basename_p;
