@@ -7,7 +7,6 @@ COPYING distributed with the source files.
 */
 
 #include <vector>
-#include <list>
 #include <cstring>
 #include <ctime>
 
@@ -30,19 +29,10 @@ COPYING distributed with the source files.
 #include <glib/gmain.h>
 
 #ifdef HAVE_OSTREAM
+#include <ios>
 #include <ostream>
 #endif
 
-// a guess for the future!
-#if GTK_MAJOR_VERSION > 2
-#define HAVE_EXPAND_TO_PATH 1
-// a pick for the present
-#elif GTK_MINOR_VERSION >= 2
-#define  HAVE_EXPAND_TO_PATH 1
-#else
-#undef HAVE_EXPAND_TO_PATH
-#endif
-
 #include "fax_list_manager.h"
 #include "fax_list_manager_icons.h"
 #include "utils/tree_path_handle.h"
@@ -64,6 +54,7 @@ extern "C" char* strptime(const char*, const char*, struct std::tm*);
 
 bool FaxListManager::fax_received_list_main_iteration = false;
 bool FaxListManager::fax_sent_list_main_iteration = false;
+GtkSortType FaxListManager::list_sort_type[FaxListEnum::sent + 1] = {GTK_SORT_ASCENDING, GTK_SORT_ASCENDING};
 
 namespace FolderListModelColumns {
   enum {icon, name, root_only, cols_num};
@@ -119,6 +110,7 @@ void FaxListManagerCB::folder_tree_view_drag_begin(GtkWidget*, GdkDragContext* c
     TreePathScopedHandle path_h(gtk_tree_model_get_path(model_p,
 							&instance_p->folder_drag_source_row_iter));
     
+    // GdkPixbufs are not owned by a GTK+ container when passed to it so use a GobjHandle
     GobjHandle<GdkPixmap> pixmap_h(gtk_tree_view_create_row_drag_icon(instance_p->folder_tree_view_p, path_h));
     GdkColormap* colormap_p = gtk_widget_get_default_colormap();
 
@@ -152,6 +144,7 @@ void FaxListManagerCB::fax_tree_view_drag_begin(GtkWidget*, GdkDragContext* cont
     if (cell_height < 0) cell_height = 0;
     if (x_pos < 0) x_pos = 0;
 
+    // GdkPixbufs are not owned by a GTK+ container when passed to it so use a GobjHandle
     GobjHandle<GdkPixmap> pixmap_h(gtk_tree_view_create_row_drag_icon(instance_p->fax_tree_view_p,
 								      rows_handle.front()));
     GdkColormap* colormap_p = gtk_widget_get_default_colormap();
@@ -317,6 +310,26 @@ gboolean FaxListManagerCB::write_path_timer_event(void* data) {
   return true; // we want a multi-shot timer
 }
 
+void FaxListManagerCB::toggle_sort_type(GtkTreeViewColumn*, void* data) {
+  FaxListManager* instance_p = static_cast<FaxListManager*>(data);
+  if (instance_p->list_sort_type[instance_p->mode] == GTK_SORT_ASCENDING) 
+    instance_p->list_sort_type[instance_p->mode] = GTK_SORT_DESCENDING;
+  else instance_p->list_sort_type[instance_p->mode] = GTK_SORT_ASCENDING;
+
+  FolderRowToFaxModelMap::const_iterator iter;
+  for (iter = instance_p->folder_row_to_fax_model_map.begin();
+       iter != instance_p->folder_row_to_fax_model_map.end();
+       ++iter) {
+    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(iter->second.get()),
+					 FaxListModelColumns::name,
+					 instance_p->list_sort_type[instance_p->mode]);
+  }
+  // this doesn't control the sort order (the call to gtk_tree_sortable_set_sort_column_id()
+  // did that) - it just sets the direction that the arrow in the tree view shows!
+  gtk_tree_view_column_set_sort_order(instance_p->fax_date_column_p,
+				      instance_p->list_sort_type[instance_p->mode]);  
+}
+
 } // anonymous namespace
 
 
@@ -343,6 +356,7 @@ FaxListManager::FaxListManager(FaxListEnum::Mode mode_):
   target_array[1].info = 0;
 
   // create icons
+  // GdkPixbufs are not owned by a GTK+ container when passed to it so use GobjHandles
   folder_icon_h = GobjHandle<GdkPixbuf>(gdk_pixbuf_new_from_xpm_data(folder_xpm));
   folder_icon_h = GobjHandle<GdkPixbuf>(gdk_pixbuf_scale_simple(folder_icon_h,
 								14, 14,
@@ -353,7 +367,7 @@ FaxListManager::FaxListManager(FaxListEnum::Mode mode_):
 							       GDK_INTERP_BILINEAR));
 
   // create the tree model for the folders and put it in a GobjHandle to handle
-  // its lifetime, because it is derived from GObject but not GtkObject
+  // its lifetime, because it is not owned by a GTK+ container
   folder_tree_store_h =
     GobjHandle<GtkTreeModel>(GTK_TREE_MODEL(gtk_tree_store_new(FolderListModelColumns::cols_num,
 							       GDK_TYPE_PIXBUF,
@@ -393,10 +407,16 @@ FaxListManager::FaxListManager(FaxListEnum::Mode mode_):
   // provide renderers for fax tree view, pack into the fax tree view columns
   // and connect to the fax tree model columns
   renderer_p = gtk_cell_renderer_text_new();
-  column_p = gtk_tree_view_column_new_with_attributes(gettext("Date"), renderer_p,
-						      "text", FaxListModelColumns::fax_date,
-						      static_cast<void*>(0));
-  gtk_tree_view_append_column(fax_tree_view_p, column_p);
+  fax_date_column_p = gtk_tree_view_column_new_with_attributes(gettext("Date"), renderer_p,
+							       "text", FaxListModelColumns::fax_date,
+							       static_cast<void*>(0));
+  // make the Date column clickable
+  gtk_tree_view_column_set_clickable(fax_date_column_p, true);
+  g_signal_connect(G_OBJECT(fax_date_column_p), "clicked",
+		   G_CALLBACK(FaxListManagerCB::toggle_sort_type), this);
+  gtk_tree_view_append_column(fax_tree_view_p, fax_date_column_p);
+
+
   renderer_p = gtk_cell_renderer_text_new();
   column_p = gtk_tree_view_column_new_with_attributes(gettext("Description"), renderer_p,
 						      "text", FaxListModelColumns::fax_description,
@@ -475,31 +495,10 @@ void FaxListManager::populate_fax_list(void) {
   }
   else {
 
-    struct dirent* direntry;
-    struct stat statinfo;
-    std::list<std::string> dir_list;
-
-    // first populate dir_list with the directory names in the faxin/faxsent subdirectories
-    while ((direntry = readdir(dir_p)) != 0) {
-      stat(direntry->d_name, &statinfo);
-      if (S_ISDIR(statinfo.st_mode)
-	  // keep the check for the "oldfax" sub-directory even though it is not used
-	  // in version 2.2.13 onwards (users may still have it lying around)
-	  && std::strcmp(direntry->d_name, "oldfax")
-	  && std::strcmp(direntry->d_name, ".")
-	  && std::strcmp(direntry->d_name, "..")
-	  && (mode == FaxListEnum::sent
-	      || std::strcmp(direntry->d_name, "current"))) {
-	dir_list.push_back(direntry->d_name);
-      }
-    }
-    while (closedir(dir_p) == -1 && errno == EINTR);
-
-    // now insert the directory names in fax_list, with description (if any)
-    // first clear the list store -- we need to clear it here before
-    // testing for an empty dir_list, or on deleting the last fax in the list,
-    // the clear won't take out the former last entry
+    // before reading the faxin/faxsent directory for faxes set up the
+    // folder items we always need
 
+    /* - we don't need to do this as the program is now written
     gtk_tree_store_clear(GTK_TREE_STORE(folder_tree_store_h.get()));
     // keep the old tree stores (if any) alive, until we have reset the model
     // in the fax tree view later in this method, by copying the values
@@ -507,6 +506,7 @@ void FaxListManager::populate_fax_list(void) {
     // now we can clear the fax list stores map
     folder_row_to_fax_model_map.clear();
     folder_name_validator.clear();
+    */
 
     // we always have an 'Inbox' and 'Sent box' row items
     GtkTreeIter base_folder_iter;
@@ -561,6 +561,8 @@ void FaxListManager::populate_fax_list(void) {
     path_to_row_map.insert(PathToFolderRowMap::value_type(trash_path, trash_row_ref_h));
 
     // now create a fax tree model for the base and trash folders
+    // put them in a GobjHandle to handle their lifetime, because they are not
+    // owned by a GTK+ container
     fax_base_model_h =
       GobjHandle<GtkTreeModel>(GTK_TREE_MODEL(gtk_list_store_new(FaxListModelColumns::cols_num,
 								 G_TYPE_STRING,
@@ -572,10 +574,10 @@ void FaxListManager::populate_fax_list(void) {
 									     G_TYPE_STRING)));
     gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(fax_base_model_h.get()),
 					 FaxListModelColumns::name,
-					 GTK_SORT_ASCENDING);
+					 list_sort_type[mode]);
     gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(trash_model_h.get()),
 					 FaxListModelColumns::name,
-					 GTK_SORT_ASCENDING);
+					 list_sort_type[mode]);
     folder_row_to_fax_model_map.insert(FolderRowToFaxModelMap::value_type(base_row_ref_h, fax_base_model_h));
     folder_row_to_fax_model_map.insert(FolderRowToFaxModelMap::value_type(trash_row_ref_h, trash_model_h));
 
@@ -596,16 +598,33 @@ void FaxListManager::populate_fax_list(void) {
       while (std::getline(file, line)) get_folders(path_to_row_map, line);
     }
 
-    // populate the tree store
-    if (!dir_list.empty()) {
-      // first sort them
-      dir_list.sort();
-      
-      std::list<std::string>::const_iterator iter;
+    // now populate the tree store
+    struct dirent* direntry;
+    struct stat statinfo;
 
-      for (iter = dir_list.begin(); iter!= dir_list.end(); ++iter) {
+    while ((direntry = readdir(dir_p)) != 0) {
+      stat(direntry->d_name, &statinfo);
+      if (S_ISDIR(statinfo.st_mode)
+	  // keep the check for the "oldfax" sub-directory even though it is not used
+	  // in version 2.2.13 onwards (users may still have it lying around)
+	  && std::strcmp(direntry->d_name, "oldfax")
+	  && std::strcmp(direntry->d_name, ".")
+	  && std::strcmp(direntry->d_name, "..")
+	  && (mode == FaxListEnum::sent
+	      || std::strcmp(direntry->d_name, "current"))) {
+	// checking for "current" will prevent double entries if completion of a received
+	// fax occurs while we are scanning the faxin directory - it is transferred to its
+	// own directory in the EfaxController::timer_event() callback and that callback
+	// cannot be invoked until this method has returned (note also the checks for
+	// FaxListManager::is_fax_received_list_main_iteration() and
+	// FaxListManager::is_fax_sent_list_main_iteration() in EfaxController::timer_event())
+	// likewise a completed sent fax will not be stored in a directory in the faxsent
+	// directory (if we are scanning that directory) until that is done via
+	// EfaxController::timer_event()
 
-	// this for loop can be time intensive as it reads from disk
+	std::string fax_name(direntry->d_name);
+
+	// this while loop can be time intensive as it reads from disk
 	// so clear any outstanding events in the main program loop
 	if (mode == FaxListEnum::received) fax_received_list_main_iteration = true;
 	else fax_sent_list_main_iteration = true;
@@ -619,7 +638,7 @@ void FaxListManager::populate_fax_list(void) {
 
 	// get the fax list Path for each fax
 	filename = dir + '/';
-	filename += *iter;
+	filename += fax_name;
 	filename += "/Path";
 
 #ifdef HAVE_IOS_NOCREATE
@@ -639,7 +658,7 @@ void FaxListManager::populate_fax_list(void) {
 	// now open up a file stream to read any fax description
 	// to pass by reference to insert_fax_on_populate()
 	filename = dir + '/';
-	filename += *iter;
+	filename += fax_name;
 	filename += "/Description";
 
 #ifdef HAVE_IOS_NOCREATE
@@ -657,18 +676,20 @@ void FaxListManager::populate_fax_list(void) {
 
 	// put the faxes in the relevant fax tree model for their folder
 	if (line.size() == 1 && line[0] == PATH_DIVIDER) {
-	  insert_fax_on_populate(path_to_row_map, file, base_path, *iter);
+	  insert_fax_on_populate(path_to_row_map, file, base_path, fax_name);
 	}
 	// include a call to get_folders() in case something has gone wrong
 	// and the fax belongs to a folder not listed in the PathList file
 	else if (!line.empty() && get_folders(path_to_row_map, line)) {
-	  insert_fax_on_populate(path_to_row_map, file, line, *iter);
+	  insert_fax_on_populate(path_to_row_map, file, line, fax_name);
 	}
 	else {
-	  insert_fax_on_populate(path_to_row_map, file, base_path, *iter);
+	  insert_fax_on_populate(path_to_row_map, file, base_path, fax_name);
 	}
       }
     }
+    // close the directory search
+    while (closedir(dir_p) == -1 && errno == EINTR);
 
     // select the main folder
     std::string base_folder;
@@ -756,14 +777,15 @@ bool FaxListManager::get_folders(PathToFolderRowMap& path_to_row_map, const std:
       row_ref_h = TreeRowRefSharedHandle(gtk_tree_row_reference_new(folder_tree_store_h,
 								    path_h));
       path_to_row_map.insert(PathToFolderRowMap::value_type(pathname, row_ref_h));
-      // create a fax tree model for the folder
+      // create a fax tree model for the folder and put it in a GobjHandle to handle its
+      // lifetime, because it is not owned by a GTK+ container
       GobjHandle<GtkTreeModel> model_h(GTK_TREE_MODEL(gtk_list_store_new(FaxListModelColumns::cols_num,
 									 G_TYPE_STRING,
 									 G_TYPE_STRING,
 									 G_TYPE_STRING)));
       gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model_h.get()),
 					   FaxListModelColumns::name,
-					   GTK_SORT_ASCENDING);
+					   list_sort_type[mode]);
       folder_row_to_fax_model_map.insert(FolderRowToFaxModelMap::value_type(row_ref_h, model_h));
     }
     else {                        // build on the nearest ancestor already created
@@ -788,14 +810,15 @@ bool FaxListManager::get_folders(PathToFolderRowMap& path_to_row_map, const std:
 	row_ref_h = TreeRowRefSharedHandle(gtk_tree_row_reference_new(folder_tree_store_h,
 								      child_path_h));
 	path_to_row_map.insert(PathToFolderRowMap::value_type(pathname, row_ref_h));
-	// create a fax tree model for the folder
+	// create a fax tree model for the folder and put it in a GobjHandle to handle its
+	// lifetime, because it is not owned by a GTK+ container
 	GobjHandle<GtkTreeModel> model_h(GTK_TREE_MODEL(gtk_list_store_new(FaxListModelColumns::cols_num,
 									   G_TYPE_STRING,
 									   G_TYPE_STRING,
 									   G_TYPE_STRING)));
 	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model_h.get()),
 					     FaxListModelColumns::name,
-					     GTK_SORT_ASCENDING);
+					     list_sort_type[mode]);
 	folder_row_to_fax_model_map.insert(FolderRowToFaxModelMap::value_type(row_ref_h, model_h));
       }
       else {
@@ -829,14 +852,15 @@ bool FaxListManager::get_folders(PathToFolderRowMap& path_to_row_map, const std:
 	row_ref_h = TreeRowRefSharedHandle(gtk_tree_row_reference_new(folder_tree_store_h,
 								      child_path_h));
 	path_to_row_map.insert(PathToFolderRowMap::value_type(pathname, row_ref_h));
-	// create a fax tree model for the folder
+	// create a fax tree model for the folder and put it in a GobjHandle to handle its
+	// lifetime, because it is not owned by a GTK+ container
 	GobjHandle<GtkTreeModel> model_h(GTK_TREE_MODEL(gtk_list_store_new(FaxListModelColumns::cols_num,
 									   G_TYPE_STRING,
 									   G_TYPE_STRING,
 									   G_TYPE_STRING)));
 	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model_h.get()),
 					     FaxListModelColumns::name,
-					     GTK_SORT_ASCENDING);
+					     list_sort_type[mode]);
 	folder_row_to_fax_model_map.insert(FolderRowToFaxModelMap::value_type(row_ref_h, model_h));
       }
       else {
@@ -1126,7 +1150,7 @@ void FaxListManager::move_folder(GtkTreeIter* dest_folder_row_iter_p,
       folder_row_to_fax_model_map.insert(FolderRowToFaxModelMap::value_type(new_row_ref_h,
 									    fax_model_h));
 
-#ifdef HAVE_EXPAND_TO_PATH
+#if GTK_CHECK_VERSION(2,2,0)
       // get the moved folder to show to its dropped level (do this before the call
       // to move_child_folders_for_level() or it may be invalidated)
       if (gtk_tree_path_up(new_path_h)) {
@@ -1266,7 +1290,16 @@ void FaxListManager::display_faxes(void) {
     write_error("Mapping error in FaxListManager::display_faxes()\n");
   }
 
-  else gtk_tree_view_set_model(fax_tree_view_p, row_map_iter->second);
+  else {
+    gtk_tree_view_set_model(fax_tree_view_p, row_map_iter->second);
+    // provide a sort indicator - we need to do it here as setting
+    // the tree model will unset the sort indicator (a bug in GTK+?)
+    gtk_tree_view_column_set_sort_indicator(fax_date_column_p, true);
+    // this doesn't control the sort order (the previous calls to
+    // gtk_tree_sortable_set_sort_column_id() on the tree models did that)
+    // - it just sets the direction that the arrow in the tree view shows!
+    gtk_tree_view_column_set_sort_order(fax_date_column_p, list_sort_type[mode]);
+  }
 }
 
 void FaxListManager::empty_trash_folder(void) {
@@ -1439,14 +1472,15 @@ void FaxListManager::make_folder(const std::string& folder_name, bool test_valid
     TreeRowRefSharedHandle row_ref_h =
       TreeRowRefSharedHandle(gtk_tree_row_reference_new(folder_tree_store_h,
 							path_h));
-    // create a fax tree model for the folder
+    // create a fax tree model for the folder and put it in a GobjHandle to handle its
+    // lifetime, because it is not owned by a GTK+ container
     GobjHandle<GtkTreeModel> model_h(GTK_TREE_MODEL(gtk_list_store_new(FaxListModelColumns::cols_num,
 								       G_TYPE_STRING,
 								       G_TYPE_STRING,
 								       G_TYPE_STRING)));
     gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model_h.get()),
 					 FaxListModelColumns::name,
-					 GTK_SORT_ASCENDING);
+					 list_sort_type[mode]);
     folder_row_to_fax_model_map.insert(FolderRowToFaxModelMap::value_type(row_ref_h, model_h));
 
     write_path();
