@@ -12,7 +12,7 @@ COPYING distributed with the source files.
 
 #include <cstdlib>
 #include <vector>
-#include <string>
+#include <cstring>
 
 #include <gtk/gtkcontainer.h>
 #include <gtk/gtktable.h>
