@@ -22,8 +22,8 @@ COPYING distributed with the source files.
 
 #include <sigc++/sigc++.h>
 
-#include "window.h"
-#include "gobj_handle.h"
+#include "utils/window.h"
+#include "utils/gobj_handle.h"
 
 
 namespace { // we put the functions in anonymous namespace in file_list.cpp
