@@ -15,7 +15,7 @@ COPYING distributed with the source files.
 #include <gtk/gtkwindow.h>
 #include <gtk/gtknotebook.h>
 
-#include "window.h"
+#include "utils/window.h"
 
 namespace { // we put the functions in anonymous namespace in helpfile.cpp
             // so they are not exported at link time
