@@ -1186,7 +1186,9 @@ int is_in_process_table(pid_t existing_lock_pid) {
     output.append(buffer, result);
   }
 
-  write(1, output.data(), output.size());
+  // for debugging
+  //write(1, output.data(), output.size());
+
   // we can safely use wait() here because we are setting the program up
   // and nothing else is calling wait() or waitpid() and we do not need
   // to worry about EINTR because there are no signals at this stage
