@@ -111,6 +111,7 @@ COPYING distributed with the source files.
 #include "utils/thread.h"
 #include "utils/fdstream.h"
 #include "utils/gerror_handle.h"
+#include "utils/callback.h"
 
 #ifdef ENABLE_NLS
 #include <libintl.h>
@@ -1818,7 +1819,8 @@ void MainWindow::start_pipe_thread(void) {
   pthread_sigmask(SIG_BLOCK, &sig_mask, 0);
 
   std::auto_ptr<Thread::Thread> res =
-    Thread::Thread::start(sigc::mem_fun(*this, &MainWindow::pipe_thread), false);
+    Thread::Thread::start(Callback::make(*this, &MainWindow::pipe_thread),
+			  false);
   if (!res.get()) {
     write_error("Cannot start new pipe thread\n");
   }
