@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2005 Chris Vine
+/* Copyright (C) 2001 to 2007 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -15,10 +15,10 @@ COPYING distributed with the source files.
 #include <sys/types.h>
 
 #include <string>
-#include <fstream>
 #include <vector>
 #include <list>
 #include <utility>
+#include <memory>
 
 #include <gtk/gtkwidget.h>
 #include <gtk/gtkwindow.h>
@@ -31,6 +31,7 @@ COPYING distributed with the source files.
 #include <gtk/gtktextmark.h>
 
 #include <gdk/gdkevents.h>
+#include <gdk/gdkgc.h>
 #include <glib/gtypes.h>
 
 #include <sigc++/sigc++.h>
@@ -43,6 +44,7 @@ COPYING distributed with the source files.
 #include "socket_list.h"
 #include "tray_icon.h"
 #include "helpfile.h"
+#include "logger.h"
 #include "utils/pipes.h"
 #include "utils/shared_ptr.h"
 #include "utils/mutex.h"
@@ -55,34 +57,23 @@ COPYING distributed with the source files.
 #define MAINWIN_SAVE_FILE ".efax-gtk_mainwin_save"
 
 
-namespace { // we put the function in anonymous namespace in mainwindow.cpp
-            // so it is not exported at link time
-namespace MessageTextCB {
-  extern "C" gboolean message_text_logfile_timer(void*);
-}
-}
-
 class MessageText: public sigc::trackable, public MainWidgetBase {
 
-  int logfile_count;
-  std::ofstream logfile;
-  std::string logfile_name;
-  std::string starting_dirname;
   GtkWidget* text_view_p;
   GtkTextMark* end_mark_p;
   GobjHandle<GtkTextTag> red_tag_h;
+  std::auto_ptr<Logger> logger_a;
+  GtkWindow* window_p;
 
-  guint timer_tag;
   void cleanify(std::string&);
-  void logfile_timer_impl(void);
 public:
-  friend gboolean MessageTextCB::message_text_logfile_timer(void*);
-
   void write_black_slot(const char* message);
   void write_red_slot(const char* message);
-  void reset_logfile(void);
-  MessageText(void);
-  ~MessageText(void);
+  void print_log(void) {logger_a->print_log();}
+  static void print_page_setup(GtkWindow* parent_p) {Logger::print_page_setup(parent_p);}
+  void view_log(const int standard_size) {logger_a->view_log(standard_size);}
+  void reset_logfile(void) {logger_a->reset_logfile();}
+  MessageText(GtkWindow* window_p);
 };
 
 class StatusLine: public sigc::trackable, public MainWidgetBase {
@@ -152,6 +143,9 @@ class MainWindow: public sigc::trackable, public WinBase {
   GtkMenuItem* address_book_item_p;
   GtkMenuItem* settings_item_p;
   GtkMenuItem* quit_item_p;
+  GtkMenuItem* print_log_item_p;
+  GtkMenuItem* page_setup_log_item_p;
+  GtkMenuItem* view_log_item_p;
   GtkMenuItem* about_efax_gtk_item_p;
   GtkMenuItem* about_efax_item_p; 
   GtkMenuItem* translations_item_p;
@@ -175,6 +169,8 @@ class MainWindow: public sigc::trackable, public WinBase {
 
   Utf8::Reassembler error_pipe_reassembler;
 
+  GobjHandle<GdkGC> red_gc_h;
+
   // these are called by handlers of GTK+ signals
   void close_impl(void);
   void get_file_impl(void);
