@@ -210,19 +210,7 @@ ReceiveMessages::ReceiveMessages(void): SettingsMessagesBase(exec + 1) {
   captions[exec] = gettext("efax-gtk help: Execute program when fax received");
 }
 
-LoggingMessages::LoggingMessages(void): SettingsMessagesBase(logfile + 1) {
-
-  messages[logfile] = gettext("This parameter specifies a log file to which progress on negotiations "
-			      "and fax status, and errors and warnings, are logged.  If none is specified, "
-			      "then no log file will be maintained.  (A log file can also be kept by "
-			      "redirecting stdout and stderr -- see the README file for further details)");
-  captions[logfile] = gettext("efax-gtk help: Log File");
-}
-
-PageMessages::PageMessages(void): SettingsMessagesBase(header + 1) {
-
-  messages[page] = gettext("Specify the page size for faxes here");
-  captions[page] = gettext("efax-gtk help: Page Size");
+SendMessages::SendMessages(void): SettingsMessagesBase(dial_prefix + 1) {
 
   messages[res] = gettext("This specifies the resolution to which faxes are sent.  \"Standard\" "
 			  "gives 204x98 and \"Fine\" gives 204x196");
@@ -233,6 +221,26 @@ PageMessages::PageMessages(void): SettingsMessagesBase(header + 1) {
 			     "want this to be shown (say because it includes a pay card access "
 			     "number)");
   captions[header] = gettext("efax-gtk help: Fax top header line");
+
+  messages[dial_prefix] = gettext("This specifies a prefix to be prepended to any dialed number - for "
+				  "example you can specify '9,' here to dial through a switchboard which "
+				  "needs 9 to obtain an outside line followed by a delay.");
+  captions[dial_prefix] = gettext("efax-gtk help: Dial Prefix");
+}
+
+LoggingMessages::LoggingMessages(void): SettingsMessagesBase(logfile + 1) {
+
+  messages[logfile] = gettext("This parameter specifies a log file to which progress on negotiations "
+			      "and fax status, and errors and warnings, are logged.  If none is specified, "
+			      "then no log file will be maintained.  (A log file can also be kept by "
+			      "redirecting stdout and stderr -- see the README file for further details)");
+  captions[logfile] = gettext("efax-gtk help: Log File");
+}
+
+PageMessages::PageMessages(void): SettingsMessagesBase(page + 1) {
+
+  messages[page] = gettext("Specify the page size for faxes here");
+  captions[page] = gettext("efax-gtk help: Page Size");
 }
 
 
