@@ -23,11 +23,11 @@ COPYING distributed with the source files.
 
 #include <sigc++/sigc++.h>
 
-#include "window.h"
-#include "gobj_handle.h"
 #include "socket_server.h"
-#include "shared_ptr.h"
-#include "mutex.h"
+#include "utils/window.h"
+#include "utils/gobj_handle.h"
+#include "utils/shared_ptr.h"
+#include "utils/mutex.h"
 
 
 namespace { // we put the functions in anonymous namespace in socket_list.cpp
@@ -56,7 +56,7 @@ class SocketListDialog: public sigc::trackable, public WinBase {
   void view_file(void);
   void remove_file_prompt(void);
   void remove_file(void);
-  std::pair<const char*, char* const*> get_view_file_parms(const string&);
+  std::pair<const char*, char* const*> get_view_file_parms(const std::string&);
   void delete_parms(std::pair<const char*, char* const*>);
 protected:
   void on_delete_event(void);
