@@ -25,6 +25,7 @@ COPYING distributed with the source files.
 
 #include "socket_server.h"
 #include "utils/shared_handle.h"
+#include "utils/callback.h"
 
 #ifdef HAVE_OSTREAM
 #include <ios>
@@ -93,10 +94,11 @@ void SocketServer::start(const std::string& port_, bool other_sock_client_addres
     sigaddset(&sig_mask, SIGHUP);
     pthread_sigmask(SIG_BLOCK, &sig_mask, 0);
 
-    // this is to keep gcc-2.95.3 and libstdc++2 happy as assigning the result
+    // temp_h is to keep gcc-2.95.3 and libstdc++2 happy as assigning the result
     // of Thread::Thread::start() to socket_thread_h incorrectly reports an error
     std::auto_ptr<Thread::Thread> temp_h =
-      Thread::Thread::start(sigc::mem_fun(*this, &SocketServer::socket_thread), true);
+      Thread::Thread::start(Callback::make(*this, &SocketServer::socket_thread),
+			    true);
     socket_thread_h = temp_h;
     if (!socket_thread_h.get()) {
       write_error("Cannot start new socket thread, fax socket will not run\n");
