@@ -49,11 +49,12 @@ modifications of the source code in this file).
   AsyncQueueDispatch is a class which has a blocking pop() method,
   which allows it to be waited on by a dedicated event/message
   dispatching thread for incoming work (represented by the data pushed
-  onto the queue).
+  onto the queue).  In the same way, it can be used to implement
+  thread pools, by having threads in the pool waiting on the queue.
 
-  By default the queues use a std::list object as its container
-  because in the kind of use mentioned above it is unlikely to hold
-  many objects but it can be changed to, say, a std::deque object by
+  By default the queues use a std::list object as their container
+  because in the kind of use mentioned above they are unlikely to hold
+  many objects but they can be changed to, say, a std::deque object by
   specifying it as the second template parameter
 */
 
