@@ -38,14 +38,15 @@ of fifo pipes.  The constructor of the class may be passed an
 enumerator to indicate whether the pipe is to read in non-blocking
 mode.
 
-The write(char*) method taking a string must have a string less than
-PIPE_BUF characters in length, or it will fail.  With the write(char*,
-int) method, taking an array and int arguments, the value of int
-(indicating the number of chars in the array to be written) should
-usually be less than PIPE_BUF, as POSIX write() guarantees that if it
-is less than PIPE_BUF, either all will be written or, if the pipe is
-too full, none will be written.  This prevents data interleaving if a
-number of independent writes to the fifo are made.
+With the write(char*, int) method, taking an array and int arguments,
+the value of int (indicating the number of chars in the array to be
+written) should usually be less than PIPE_BUF, as POSIX write()
+guarantees that if it is less than PIPE_BUF, either all will be
+written or, if the pipe is too full, none will be written.  This
+prevents data interleaving if a number of independent writes to the
+fifo are made.  The same is true of the write (char*) method taking
+a string - if the string is longer than PIPE_BUF in size, data
+interleaving may occur.  
 
 The pipe is set to read in non-blocking mode if the constructor or
 PipeFifo is passed the value PipeFifo::non_block.  If constructed
@@ -57,8 +58,8 @@ sparingly -- when set in this way write() will always try to write
 something.  The atomic guarantees mentioned in the preceding paragraph
 do not apply and data can be interleaved or lost.
 
-PIPE_BUF is defined in limits.h, and is at least 512 bytes, and usually 4096 bytes,
-but may be calculated from other files included by limits.h.
+PIPE_BUF is defined in limits.h, and is at least 512 bytes, and usually
+4096 bytes, but may be calculated from other files included by limits.h.
 
 Where a pipe is used to communicate between parent and child after a
 call to fork(), the pipe will normally be used unidirectionally unless
@@ -71,7 +72,7 @@ process concerned calling the methods PipeFifo::make_writeonly() or
 PipeFifo::make_readonly() after the fork.  If an attempt is made to
 read or write to a descriptor closed in this way, the
 PipeFifo::read() or PipeFifo::write() method will ensure that no
-read or write will take place, and instead a -1 will be returned.
+read or write will take place, and instead -2 will be returned.
 
 The methods PipeFifo::connect_to_stdin(), PipeFifo::connect_to_stdout()
 and PipeFifo::connect_to_stderr() are available to be used in the child
