@@ -40,7 +40,26 @@ modifications of the source code in this file).
 #include <cstdlib>
 #include <glib/gmem.h>
 
+// define this if you want to use GLIB atomic functions/memory barriers
+// instead of a (slower) mutex to lock the reference count in the
+// SharedLockHandle class - to do this at least glib version 2.4 is
+// required
+#define SHARED_LOCK_HANDLE_USE_GLIB_ATOMIC_OPS 1
+
+
+#ifdef SHARED_LOCK_HANDLE_USE_GLIB_ATOMIC_OPS
+#include <glib/gutils.h>
+#  if GLIB_CHECK_VERSION(2,4,0)
+#  include <glib/gtypes.h>
+#  include <glib/gatomic.h>
+#  else
+#  undef SHARED_LOCK_HANDLE_USE_GLIB_ATOMIC_OPS
+#  endif
+#endif
+
+#ifndef SHARED_LOCK_HANDLE_USE_GLIB_ATOMIC_OPS
 #include "mutex.h"
+#endif
 
 /*
 The SharedHandle class is similar to the SharedPtr class (it keeps a
@@ -234,15 +253,14 @@ public:
 
 /*
   Class SharedLockHandle is a version of the shared handle class which
-  includes mutex locking so that it can be accessed in multiple
-  threads. Note that only the reference count is protected by a mutex,
-  so this is thread safe in the sense in which a raw pointer is thread
-  safe.  A shared handle accessed in one thread referencing a
-  particular object is thread safe as against another shared handle
-  accessing the same object in a different thread.  It is thus
-  suitable for use in different Std C++ containers which exist in
-  different threads but which contain shared objects by reference.
-  But:
+  includes locking so that it can be accessed in multiple threads.
+  Note that only the reference count is protected, so this is thread
+  safe in the sense in which a raw pointer is thread safe.  A shared
+  handle accessed in one thread referencing a particular object is
+  thread safe as against another shared handle accessing the same
+  object in a different thread.  It is thus suitable for use in
+  different Std C++ containers which exist in different threads but
+  which contain shared objects by reference.  But:
 
   1.  If the referenced object is to be modified in one thread and
       read or modified in another thread an appropriate mutex for the
@@ -271,8 +289,12 @@ template <class T, class Dealloc = StandardArrayDelete<T> > class SharedLockHand
   Dealloc deleter;
 
   struct Ref_items {
+#ifndef SHARED_LOCK_HANDLE_USE_GLIB_ATOMIC_OPS
     Thread::Mutex* mutex_p;
     unsigned int* ref_count_p;
+#else
+    gint* ref_count_p;
+#endif
     void* obj_p;
   } ref_items;
 
@@ -280,6 +302,7 @@ template <class T, class Dealloc = StandardArrayDelete<T> > class SharedLockHand
   // because  Thread::Mutex::~Mutex(), Thread::Mutex::lock() and Thread::Mutex::unlock()
   // do not throw
   void unreference(void) {
+#ifndef SHARED_LOCK_HANDLE_USE_GLIB_ATOMIC_OPS
     ref_items.mutex_p->lock();
     --(*ref_items.ref_count_p);
     if (*ref_items.ref_count_p == 0) {
@@ -289,18 +312,29 @@ template <class T, class Dealloc = StandardArrayDelete<T> > class SharedLockHand
       deleter(static_cast<T>(ref_items.obj_p));
     }
     else ref_items.mutex_p->unlock();
+#else
+    if (g_atomic_int_dec_and_test(ref_items.ref_count_p)) {
+      delete ref_items.ref_count_p;
+      deleter(static_cast<T>(ref_items.obj_p));
+    }
+#endif
   }
   
   // SharedLockHandle<T, Dealloc>::reference() does not throw exceptions because
   // Thread::Mutex::Lock::Lock() and Thread::Mutex::Lock::~Lock() do not throw
   void reference(void) {
+#ifndef SHARED_LOCK_HANDLE_USE_GLIB_ATOMIC_OPS
     Thread::Mutex::Lock lock(*ref_items.mutex_p);
     ++(*ref_items.ref_count_p);
+#else
+    g_atomic_int_inc(ref_items.ref_count_p);
+#endif
   }
 
 public:
   // constructor of first SharedLockHandle holding the referenced object
   explicit SharedLockHandle(T ptr = 0) {
+#ifndef SHARED_LOCK_HANDLE_USE_GLIB_ATOMIC_OPS
     try {
       ref_items.mutex_p = new Thread::Mutex;
     }
@@ -318,6 +352,17 @@ public:
       deleter(ptr);
       throw;
     }
+#else
+    try {
+      ref_items.ref_count_p = new gint(1);
+    }
+    catch (...) {
+      deleter(ptr); // if allocating the int referenced by ref_items.ref_count_p
+                    // has failed then delete the object to be referenced to
+                    // avoid a memory leak
+      throw;
+    }
+#endif
     ref_items.obj_p = ptr;
   }
 
@@ -356,8 +401,12 @@ public:
   T get(void) const {return static_cast<T>(ref_items.obj_p);}
   operator T(void) const {return static_cast<T>(ref_items.obj_p);}
   unsigned int get_refcount(void) const {
+#ifndef SHARED_LOCK_HANDLE_USE_GLIB_ATOMIC_OPS
     Thread::Mutex::Lock lock(*ref_items.mutex_p);
     return *ref_items.ref_count_p;
+#else
+    return g_atomic_int_get(ref_items.ref_count_p);
+#endif
   }
 
   // destructor
