@@ -37,7 +37,25 @@ modifications of the source code in this file).
 #ifndef SHARED_PTR_H
 #define SHARED_PTR_H
 
+// define this if you want to use GLIB atomic functions/memory barriers
+// instead of a (slower) mutex to lock the reference count in the
+// SharedLockPtr class - to do this at least glib version 2.4 is required
+#define SHARED_LOCK_PTR_USE_GLIB_ATOMIC_OPS 1
+
+
+#ifdef SHARED_LOCK_PTR_USE_GLIB_ATOMIC_OPS
+#include <glib/gutils.h>
+#  if GLIB_CHECK_VERSION(2,4,0)
+#  include <glib/gtypes.h>
+#  include <glib/gatomic.h>
+#  else
+#  undef SHARED_LOCK_PTR_USE_GLIB_ATOMIC_OPS
+#  endif
+#endif
+
+#ifndef SHARED_LOCK_PTR_USE_GLIB_ATOMIC_OPS
 #include "mutex.h"
+#endif
 
 template <class T> class SharedPtr {
 
@@ -134,15 +152,14 @@ public:
 
 /*
   Class SharedLockPtr is a version of the shared pointer class which
-  includes mutex locking so that it can be accessed in multiple
-  threads. Note that only the reference count is protected by a mutex,
-  so this is thread safe in the sense in which a raw pointer is thread
-  safe.  A shared pointer accessed in one thread referencing a
-  particular object is thread safe as against another shared pointer
-  accessing the same object in a different thread.  It is thus
-  suitable for use in different Std C++ containers which exist in
-  different threads but which contain shared objects by reference.
-  But:
+  includes locking so that it can be accessed in multiple threads.
+  Note that only the reference count is protected, so this is thread
+  safe in the sense in which a raw pointer is thread safe.  A shared
+  pointer accessed in one thread referencing a particular object is
+  thread safe as against another shared pointer accessing the same
+  object in a different thread.  It is thus suitable for use in
+  different Std C++ containers which exist in different threads but
+  which contain shared objects by reference.  But:
 
   1.  If the referenced object is to be modified in one thread and
       read or modified in another thread an appropriate mutex for the
@@ -169,8 +186,12 @@ public:
 template <class T> class SharedLockPtr {
 
   struct {
+#ifndef SHARED_LOCK_PTR_USE_GLIB_ATOMIC_OPS
     Thread::Mutex* mutex_p;
     unsigned int* ref_count_p;
+#else
+    gint* ref_count_p;
+#endif
     T* obj_p;
   } ref_items;
 
@@ -178,6 +199,7 @@ template <class T> class SharedLockPtr {
   // contained object does not throw, because  Thread::Mutex::~Mutex(),
   // Thread::Mutex::lock() and Thread::Mutex::unlock() do not throw
   void unreference(void) {
+#ifndef SHARED_LOCK_PTR_USE_GLIB_ATOMIC_OPS
     ref_items.mutex_p->lock();
     --(*ref_items.ref_count_p);
     if (*ref_items.ref_count_p == 0) {
@@ -187,18 +209,29 @@ template <class T> class SharedLockPtr {
       delete ref_items.obj_p;  // if this is NULL it is still safe to apply delete to it
     }
     else ref_items.mutex_p->unlock();
+#else
+    if (g_atomic_int_dec_and_test(ref_items.ref_count_p)) {
+      delete ref_items.ref_count_p;
+      delete ref_items.obj_p;  // if this is NULL it is still safe to apply delete to it
+    }
+#endif
   }
 
   // SharedLockPtr<T>::reference() does not throw because
   // Thread::Mutex::Lock::Lock() and Thread::Mutex::Lock::~Lock() do not throw
   void reference(void) {
+#ifndef SHARED_LOCK_PTR_USE_GLIB_ATOMIC_OPS
     Thread::Mutex::Lock lock(*ref_items.mutex_p);
     ++(*ref_items.ref_count_p);
+#else
+    g_atomic_int_inc(ref_items.ref_count_p);
+#endif
   }
 
 public:
   // constructor of first SharedLockPtr holding the referenced object
   explicit SharedLockPtr(T* ptr = 0) {
+#ifndef SHARED_LOCK_PTR_USE_GLIB_ATOMIC_OPS
     try {
       ref_items.mutex_p = new Thread::Mutex;
     }
@@ -216,6 +249,17 @@ public:
       delete ptr;
       throw;
     }
+#else
+    try {
+      ref_items.ref_count_p = new gint(1);
+    }
+    catch (...) {
+      delete ptr;  // if allocating the int referenced by ref_items.ref_count_p
+                   // has failed then delete the object to be referenced to
+                   // avoid a memory leak
+      throw;
+    }
+#endif
     ref_items.obj_p = ptr;
   }
 
@@ -233,7 +277,9 @@ public:
     // because we are allowing an implicit cast from derived to
     // base class referenced object, we need to assign from each
     // member of sh_ptr.ref_items separately
+#ifndef SHARED_LOCK_PTR_USE_GLIB_ATOMIC_OPS
     ref_items.mutex_p = sh_ptr.ref_items.mutex_p;
+#endif
     ref_items.obj_p = sh_ptr.ref_items.obj_p;
     ref_items.ref_count_p = sh_ptr.ref_items.ref_count_p;
     reference();
@@ -272,8 +318,12 @@ public:
   T& operator*(void) const {return *ref_items.obj_p;}
   T* operator->(void) const {return ref_items.obj_p;}
   unsigned int get_refcount(void) const {
+#ifndef SHARED_LOCK_PTR_USE_GLIB_ATOMIC_OPS
     Thread::Mutex::Lock lock(*ref_items.mutex_p);
     return *ref_items.ref_count_p;
+#else
+    return g_atomic_int_get(ref_items.ref_count_p);
+#endif
   }
 
   // destructor
