@@ -1,4 +1,4 @@
-/* Copyright (C) 2005 Chris Vine
+/* Copyright (C) 2005 to 2008 Chris Vine
 
 The library comprised in this file or of which this file is part is
 distributed by Chris Vine under the GNU Lesser General Public
@@ -25,24 +25,14 @@ License as follows:
 #ifndef THREAD_H
 #define THREAD_H
 
-#include "prog_defs.h"      // for sigc::slot
-
 #include <memory>
 
 #include <pthread.h>
-#include <sigc++/sigc++.h>
 
-namespace { // this function is placed in anonymous namespace in thread.ccp
-            // so that is is not exported at link time
-namespace ThreadCB {
-  extern "C" void* thread_func(void*);
-}
-
-} // anonymous namespace
+#include "callback.h"
 
 namespace Thread {
 
-
 class Thread {
   pthread_t thread;
   // private constructor - this class can only be created with Thread::start
@@ -51,7 +41,6 @@ class Thread {
   Thread(const Thread&);
   Thread& operator=(const Thread&);
 public:
-  friend void* ThreadCB::thread_func(void*);
 
   // Use Thread::cancel() with great care - sometimes its use is unavoidable
   // but destructors for local objects may not be called if a thread exits
@@ -66,22 +55,25 @@ public:
 
   void join(void) {pthread_join(thread, 0);}
 
-  // threads have single ownership semantics - the thread will keep running even
-  // if the return value of Thread::start() goes out of scope (but it will no longer
-  // be possible to call any of the methods in this class for it, which is fine if
-  // the thread is not started as joinable and it is not intended to cancel it).
-  // Thread::start() will return an empty std::autoptr<Thread> object (where
-  // std::autoptr<Thread>::get() will return 0) if the thread does not start
-  // correctly.  As Thread::start() creates a new Thread object on freestore, it
-  // will throw if the new operator throws when failing (by default it will throw
-  // std::bad_alloc on failure to allocate memory), but it will throw no other
-  // exception.  The slot object is copied from the thread starting the new thread
-  // to the new thread in thread_func(), and a semaphore in the new thread releases
-  // (post()s) and the original thread acquires (wait()s on) the semaphore.  A
-  // std::auto_ptr<> data object can therefore be bound to the slot with sigc::bind
-  // and the change of ownership will be visible to the original thread if the system
-  // is POSIX compliant with respect to memory visibility between threads
-  static std::auto_ptr<Thread> start(const sigc::slot<void>& cb, bool joinable);
+  // threads have single ownership semantics - the thread will keep
+  // running even if the return value of Thread::start() goes out of
+  // scope (but it will no longer be possible to call any of the
+  // methods in this class for it, which is fine if the thread is not
+  // started as joinable and it is not intended to cancel it).
+  // Thread::start() will return an empty std::autoptr<Thread> object
+  // (where std::autoptr<Thread>::get() will return 0) if the thread
+  // does not start correctly.  As Thread::start() creates a new
+  // Thread object on freestore, it will throw if the new operator
+  // throws when failing (by default it will throw std::bad_alloc on
+  // failure to allocate memory), but it will throw no other
+  // exception.  The first argument should be a Callback object
+  // constructed on freestore with the new expression, such as by
+  // using one of the convenience callback() functions in callback.h.
+  // Thread::start() will take ownership of the callback object - it
+  // will automatically be deleted either by the new thread when it
+  // has finished with it, or by Thread::start() in this thread if the
+  // attempt to start a new thread fails
+  static std::auto_ptr<Thread> start(const Callback::Callback* cb, bool joinable);
 };
 
 class CancelBlock {
@@ -97,6 +89,14 @@ public:
   ~CancelBlock(void) {unblock();}
 };
 
+// this class can be thrown (instead of calling pthread_exit()) when a thread
+// wishes to terminate itself and also ensure stack unwinding, so that destructors
+// of local objects are called.  It is caught automatically by the implementation
+// of Thread::start() so that it will only terminate the thread throwing it and not
+// the whole process.  See the Thread::cancel() method above, for use when a thread
+// wishes to terminate another one, and the caveats on the use of Thread::cancel().
+class Exit {};
+
 } // namespace Thread
 
 #endif
