@@ -33,8 +33,8 @@ License as follows:
    it is convenient for a class to encapsulate only part of the
    implementation of a top level widget, namely in a case where the
    main widget represented by that encapsulation is intended to be
-   placed in a GTK+ container object maintained by another class which
-   is a child of the top level window..
+   placed in a GTK+ container object maintained by another class
+   encapsulating the top level widget.
 
    The MainWidgetBase class is intended to assist this, by managing
    references with g_object_ref() and gtk_object_sink() (for GTK+-2.8
